''**************************************
''
''  WIZnet Test Driver Ver. 00.1
''
''  Timothy D. Swieter, E.I.
''  company name to go here
''
''  Updated: December 6, 2007
''
''Description:
''This program is a driver for controlling the WIZnet W5100 
''
''reference:
''      W5100_Datasheet_v1_1_2.pdf (Datasheet for chip)
''      2711015Eady.pdf (Circuit Cellar Magazine Article)
''
''**************************************

CON
'***************************************
'  System Definitions      
'***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3v DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0v DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode
  
'***************************************
'  W5100 Common Register Definitions
'***************************************
  _MR           = $0000         'Mode Register
  _GAR0         = $0001         'Gateway Address Register
  _GAR1         = $0002
  _GAR2         = $0003
  _GAR3         = $0004
  _SUBR0        = $0005         'Subnet Mask Address Register
  _SUBR1        = $0006
  _SUBR2        = $0007
  _SUBR3        = $0008
  _SHAR0        = $0009         'Source Hardware Address Register (MAC)
  _SHAR1        = $000A
  _SHAR2        = $000B
  _SHAR3        = $000C
  _SHAR4        = $000D
  _SHAR5        = $000E
  _SIPR0        = $000F         'Source IP Address Register
  _SIPR1        = $0010
  _SIPR2        = $0011
  _SIPR3        = $0012
  'Reserved space $0013 - $0014
  _IR           = $0015         'Interrupt Register
  _IMR          = $0016         'Interrupt Mask Register
  _RTR0         = $0017         'Retry Time Register
  _RTR1         = $0018
  _RCR          = $0019         'Retry Count Register
  _RMSR         = $001A         'Rx Memory Size Register
  _TMSR         = $001B         'Tx Memory Size Register
  _PATR0        = $001C         'Authentication Type in PPPoE Register
  _PATR1        = $001D
  'Reserved space $001E - $0027
  _PTIMER       = $0028         'PPP LCP Request Timer
  _PMAGIC       = $0029         'PPP LCP Magic Number
  _UIPR0        = $002A         'Unreachable IP Address Register
  _UIPR1        = $002B
  _UIPR2        = $002C
  _UIPR3        = $002D
  _UPORT0       = $002E         'Unreachable Port Register
  _UPORT1       = $002F
  'Reserved space $0030 - $03FF

'***************************************
'  W5100 Socket 0 Register Definitions
'***************************************
  _S0_MR        = $0400         'Socket 0 Mode Register
  _S0_CR        = $0401         'Socket 0 Command Register
  _S0_IR        = $0402         'Socket 0 Interrupt Register
  _S0_SR        = $0403         'Socket 0 Status Register
  _S0_PORT0     = $0404         'Socket 0 Source Port Register
  _S0_PORT1     = $0405
  _S0_DHAR0     = $0406         'Socket 0 Destination Hardware Address Register
  _S0_DHAR1     = $0407
  _S0_DHAR2     = $0408
  _S0_DHAR3     = $0409
  _S0_DHAR4     = $040A
  _S0_DHAR5     = $040B
  _S0_DIPR0     = $040C         'Socket 0 Destination IP Address Register
  _S0_DIPR1     = $040D
  _S0_DIPR2     = $040E
  _S0_DIPR3     = $040F
  _S0_DPORT0    = $0410         'Socket 0 Destination Port Register
  _S0_DPORT1    = $0411
  _S0_MSSR0     = $0412         'Socket 0 Maximum Segment Size Register
  _S0_MSSR1     = $0413
  _S0_PROTO     = $0414         'Socket 0 Protocol in IP Raw Mode Register
  _S0_TOS       = $0415         'Socket 0 IP TOS Register
  _S0_TTL       = $0416         'Socket 0 IP TTL Register
  'Reserved space $0417 - $041F
  _S0_TX_FSRO   = $0420         'Socket 0 TX Free Size Register
  _S0_TX_FSR1   = $0421
  _S0_TX_RD0    = $0422         'Socket 0 TX Read Pointer Register
  _S0_TX_RD1    = $0423
  _S0_TX_WR0    = $0424         'Socket 0 TX Write Pointer Register
  _S0_TX_WR1    = $0425
  _S0_RX_RSR0   = $0426         'Socket 0 RX Received Size Register
  _S0_RX_RSR1   = $0427
  _S0_RX_RD0    = $0428         'Socket 0 RX Read Pointer Register
  _S0_RX_RD1    = $0429
  'Reserved space $042A - $04FF

'***************************************
'  W5100 Socket 1 Register Definitions
'***************************************
  _S1_MR        = $0500         'Socket 1 Mode Register
  _S1_CR        = $0501         'Socket 1 Command Register
  _S1_IR        = $0502         'Socket 1 Interrupt Register
  _S1_SR        = $0503         'Socket 1 Status Register
  _S1_PORT0     = $0504         'Socket 1 Source Port Register
  _S1_PORT1     = $0505
  _S1_DHAR0     = $0506         'Socket 1 Destination Hardware Address Register
  _S1_DHAR1     = $0507
  _S1_DHAR2     = $0508
  _S1_DHAR3     = $0509
  _S1_DHAR4     = $050A
  _S1_DHAR5     = $050B
  _S1_DIPR0     = $050C         'Socket 1 Destination IP Address Register
  _S1_DIPR1     = $050D
  _S1_DIPR2     = $050E
  _S1_DIPR3     = $050F
  _S1_DPORT0    = $0510         'Socket 1 Destination Port Register
  _S1_DPORT1    = $0511
  _S1_MSSR0     = $0512         'Socket 1 Maximum Segment Size Register
  _S1_MSSR1     = $0513
  _S1_PROTO     = $0514         'Socket 1 Protocol in IP Raw Mode Register
  _S1_TOS       = $0515         'Socket 1 IP TOS Register
  _S1_TTL       = $0516         'Socket 1 IP TTL Register
  'Reserved space $0517 - $051F
  _S1_TX_FSRO   = $0520         'Socket 1 TX Free Size Register
  _S1_TX_FSR1   = $0521
  _S1_TX_RD0    = $0522         'Socket 1 TX Read Pointer Register
  _S1_TX_RD1    = $0523
  _S1_TX_WR0    = $0524         'Socket 1 TX Write Pointer Register
  _S1_TX_WR1    = $0525
  _S1_RX_RSR0   = $0526         'Socket 1 RX Received Size Register
  _S1_RX_RSR1   = $0527
  _S1_RX_RD0    = $0528         'Socket 1 RX Read Pointer Register
  _S1_RX_RD1    = $0529
  'Reserved space $052A - $05FF

'***************************************
'  W5100 Socket 2 Register Definitions
'***************************************
  _S2_MR        = $0600         'Socket 2 Mode Register
  _S2_CR        = $0601         'Socket 2 Command Register
  _S2_IR        = $0602         'Socket 2 Interrupt Register
  _S2_SR        = $0603         'Socket 2 Status Register
  _S2_PORT0     = $0604         'Socket 2 Source Port Register
  _S2_PORT1     = $0605
  _S2_DHAR0     = $0606         'Socket 2 Destination Hardware Address Register
  _S2_DHAR1     = $0607
  _S2_DHAR2     = $0608
  _S2_DHAR3     = $0609
  _S2_DHAR4     = $060A
  _S2_DHAR5     = $060B
  _S2_DIPR0     = $060C         'Socket 2 Destination IP Address Register
  _S2_DIPR1     = $060D
  _S2_DIPR2     = $060E
  _S2_DIPR3     = $060F
  _S2_DPORT0    = $0610         'Socket 2 Destination Port Register
  _S2_DPORT1    = $0611
  _S2_MSSR0     = $0612         'Socket 2 Maximum Segment Size Register
  _S2_MSSR1     = $0613
  _S2_PROTO     = $0614         'Socket 2 Protocol in IP Raw Mode Register
  _S2_TOS       = $0615         'Socket 2 IP TOS Register
  _S2_TTL       = $0616         'Socket 2 IP TTL Register
  'Reserved space $0617 - $061F
  _S2_TX_FSRO   = $0620         'Socket 2 TX Free Size Register
  _S2_TX_FSR1   = $0621
  _S2_TX_RD0    = $0622         'Socket 2 TX Read Pointer Register
  _S2_TX_RD1    = $0623
  _S2_TX_WR0    = $0624         'Socket 2 TX Write Pointer Register
  _S2_TX_WR1    = $0625
  _S2_RX_RSR0   = $0626         'Socket 2 RX Received Size Register
  _S2_RX_RSR1   = $0627
  _S2_RX_RD0    = $0628         'Socket 2 RX Read Pointer Register
  _S2_RX_RD1    = $0629
  'Reserved space $062A - $06FF

'***************************************
'  W5100 Socket 3 Register Definitions
'***************************************
  _S3_MR        = $0700         'Socket 3 Mode Register
  _S3_CR        = $0701         'Socket 3 Command Register
  _S3_IR        = $0702         'Socket 3 Interrupt Register
  _S3_SR        = $0703         'Socket 3 Status Register
  _S3_PORT0     = $0704         'Socket 3 Source Port Register
  _S3_PORT1     = $0705
  _S3_DHAR0     = $0706         'Socket 3 Destination Hardware Address Register
  _S3_DHAR1     = $0707
  _S3_DHAR2     = $0708
  _S3_DHAR3     = $0709
  _S3_DHAR4     = $070A
  _S3_DHAR5     = $070B
  _S3_DIPR0     = $070C         'Socket 3 Destination IP Address Register
  _S3_DIPR1     = $070D
  _S3_DIPR2     = $070E
  _S3_DIPR3     = $070F
  _S3_DPORT0    = $0710         'Socket 3 Destination Port Register
  _S3_DPORT1    = $0711
  _S3_MSSR0     = $0712         'Socket 3 Maximum Segment Size Register
  _S3_MSSR1     = $0713
  _S3_PROTO     = $0714         'Socket 3 Protocol in IP Raw Mode Register
  _S3_TOS       = $0715         'Socket 3 IP TOS Register
  _S3_TTL       = $0716         'Socket 3 IP TTL Register
  'Reserved space $0717 - $071F
  _S3_TX_FSRO   = $0720         'Socket 3 TX Free Size Register
  _S3_TX_FSR1   = $0721
  _S3_TX_RD0    = $0722         'Socket 3 TX Read Pointer Register
  _S3_TX_RD1    = $0723
  _S3_TX_WR0    = $0724         'Socket 3 TX Write Pointer Register
  _S3_TX_WR1    = $0725
  _S3_RX_RSR0   = $0726         'Socket 3 RX Received Size Register
  _S3_RX_RSR1   = $0727
  _S3_RX_RD0    = $0728         'Socket 3 RX Read Pointer Register
  _S3_RX_RD1    = $0729
  'Reserved space $072A - $07FF

'***************************************
'  W5100 Register Masks & Values Defintions
'***************************************

  'Used in the mode register (MR)
  _RSTMODE      = %1000_0000    'If 1, internal registers are initialized
  _PBMODE       = %0001_0000    'Ping block mode, 1 is enabled
  _PPPOEMODE    = %0000_1000    'PPPoE mode, 1 is enabled
  _AIMODE       = %0000_0010    'Address auto-increment mode
  _INDMODE      = %0000_0001    'Indirect bus mode

  'Used in the Interrupt Register (IR) & Interrupt Mask Register (IMR)
  _CONFLICTM    = %1000_0000
  _UNREACHM     = %0100_0000
  _PPPoEM       = %0010_0000
  _S3_INTM      = %0000_1000    'Socket 3 interrupt bit mask (1 = interrupt)
  _S2_INTM      = %0000_0100    'Socket 2 interrupt bit mask (1 = interrupt)
  _S1_INTM      = %0000_0010    'Socket 1 interrupt bit mask (1 = interrupt)
  _S0_INTM      = %0000_0001    'Socket 0 interrupt bit mask (1 = interrupt)

  'Used in the RX memory size register(RMSR)
  _S3_SM        = %1100_0000    'Socket 3 size mask
  _S2_SM        = %0011_0000    'Socket 2 size mask
  _S1_SM        = %0000_1100    'Socket 1 size mask
  _S0_SM        = %0000_0011    'Socket 0 size mask

  _1KB          = %00           '1KB memory size
  _2KB          = %01           '2KB memory size
  _4KB          = %10           '4KB memory size
  _8KB          = %11           '8KB memory size

  'Used in the socket n mode register (Sn_MR)
  _MULTIM       = %1000_0000    'Enable/disable multicasting in UDP
  _PROTOCOLM    = %0000_1111    'Registers for setting protocol

  _CLOSEDPROTO  = %0000         'Closed
  _TCPPROTO     = %0001         'TCP
  _UDPPROTO     = %0010         'UDP
  _IPRAWPROTO   = %0011         'IPRAW
  _MACRAW       = %0100         'MACRAW (used in socket 0)
  _PPPOEPROTO   = %0101         'PPPoE (used in socket 0)
  
  'Used in the socket n command register (Sn_CR)
  _OPEN         = $01           'Initialize a socket
  _LISTEN       = $02           'In TCP mode, waits for request from client
  _CONNECT      = $04           'In TCP mode, sends connect request to server
  _DISCON       = $08           'In TCP mode, request to disconnect
  _CLOSE        = $10           'Closes socket
  _SEND         = $20           'Transmits data
  _SEND_MAC     = $21           'In UDP mode, like send, but uses MAC
  _SEND_KEEP    = $22           'In TCP mode, check connection status by sending 1 byte
  _RECV         = $40           'Receiving is processed

  'Used in socket n interrupt register (Sn_IR)
  _SEND_OKM     = %0001_0000    'Set to 1 if send operation is completed
  _TIMEOUTM     = %0000_1000    'Set to 1 if timeout occured during transmission
  _RECVM        = %0000_0100    'Set to 1 if data is received
  _DISCONM      = %0000_0010    'Set to 1 if connection termination is requested
  _CONM         = %0000_0001    'Set to 1 if connection is established

  'Used in socket n status register (Sn_SR)
  _SOCK_CLOSED  = $00
  _SOCK_INIT    = $13
  _SOCK_LISTEN  = $14
  _SOCK_ESTAB   = $17
  _SOCK_CLOSE_WT = $1C
  _SOCK_UDP     = $22
  _SOCK_IPRAW   = $32
  _SOCK_MACRAW  = $42
  _SOCK_PPPOE   = $5F
  _SOCK_SYNSENT = $15
  _SOCK_SYNRECV = $16
  _SOCK_FIN_WAIT = $18
  _SOCK_CLOSING = $1A
  _SOCK_TIME_WAIT = $1B
  _SOCK_LAST_ACK = $1D
  _SOCK_ARP1    = $11
  _SOCK_ARP2    = $21
  _SOCK_ARP3    = $31

  'Used in SPI mode
  _WRITEOP      = %1111_0000    'Signals a write operation in SPI mode
  _READOP       = %0000_1111    'Signals a read operation in SPI mode

VAR               'Variables to be located here

  'Addresses
  byte  GatewayAddress[4]       '4 element array containing address ex. "192.168.0.1"
  byte  SubnetMask[4]           '4 element array contianing subnet mask ex. "255.255.255.0"
  byte  MACAddress[6]           '6 element array contianing MAC or source hardware address ex. "00.08.DC.01.02.03"
  byte  IPAddress[4]            '4 element array containing IP address ex. "192.168.0.13"

  long  txrxlong                'Long used for talking to WIZnet in SPI routine
  
  long  SSpin                   'Transmission enable, 0 is transmitting, output
  long  SCLKpin                 'Serial clock pin, output
  long  MOSIpin                 'Master out slave in pin, output
  long  MISOpin                 'Master in slave out pin, input
  
                                 

OBJ               'Object declaration to be located here
'None at this time


PUB InitWIZnet(_ss, _sclk, _mosi, _miso)
''Initializes the IO and registers based on parameters
''After initilization another cog is started with the
''program to watch for tx/rx data and respond properly

  'Initialize the I/O direction and state
  'This routine assumes SPI connection, SPI_EN should be tied high on W5100
  SSpin := _ss
  DIRA[SSpin] := _OUTPUT
  OUTA[SSpin] := _HIGH

  SCLKpin := _sclk
  DIRA[SCLKpin] := _OUTPUT
  OUTA[SCLKpin] := _LOW

  MOSIpin := _mosi
  DIRA[MOSIpin] := _OUTPUT
  OUTA[MOSIpin] := _LOW

  MISOpin := _miso
  DIRA[MISOpin] := _INPUT

PUB ResetWIZnet
''Perform a software reset of the WIZnet

  WriteWizReg(_MR,_RSTMODE)

PUB PingBlockDisable | temp0
''Turn off the ping block

  temp0 := ReadWizReg(_MR)
  temp0 := temp0 & (!_PBMODE)
  
  WriteWizReg(_MR, temp0)  

PUB SetGatewayAddress(_addbyte0, _addbyte1, _addbyte2, _addbyte3)
''Writes the specified gateway address to the WIZnet

  GatewayAddress[0] := _addbyte0
  GatewayAddress[1] := _addbyte1
  GatewayAddress[2] := _addbyte2
  GatewayAddress[3] := _addbyte3

  'Write data to WIZnet
  WriteWizReg(_GAR0,GatewayAddress[0])
  WriteWizReg(_GAR1,GatewayAddress[1])
  WriteWizReg(_GAR2,GatewayAddress[2])
  WriteWizReg(_GAR3,GatewayAddress[3])

PUB ReadGatewayAddress(_addbyte) : addvalue
''Reads the specified gateway address byte out of WIZnet

  'Read data from WIZnet
  case _addbyte
    0:  addvalue := ReadWizReg(_GAR0)
    1:  addvalue := ReadWizReg(_GAR1)
    2:  addvalue := ReadWizReg(_GAR2)
    3:  addvalue := ReadWizReg(_GAR3)

  return addvalue
  
PUB SetMACAddress(_addbyte0, _addbyte1, _addbyte2, _addbyte3, _addbyte4, _addbyte5)
''Writes the specified MAC address to the WIZnet

  MACAddress[0] := _addbyte0
  MACAddress[1] := _addbyte1
  MACAddress[2] := _addbyte2
  MACAddress[3] := _addbyte3
  MACAddress[4] := _addbyte4
  MACAddress[5] := _addbyte5     

  'Write data to WIZnet
  WriteWizReg(_SHAR0,MACAddress[0])
  WriteWizReg(_SHAR1,MACAddress[1])
  WriteWizReg(_SHAR2,MACAddress[2])
  WriteWizReg(_SHAR3,MACAddress[3])
  WriteWizReg(_SHAR4,MACAddress[4])
  WriteWizReg(_SHAR5,MACAddress[5])  

PUB ReadMACAddress(_addbyte) : addvalue
''Reads the specified MAC address byte out of WIZnet

  'Read data from WIZnet
  case _addbyte
    0:  addvalue := ReadWizReg(_SHAR0)
    1:  addvalue := ReadWizReg(_SHAR1)
    2:  addvalue := ReadWizReg(_SHAR2)
    3:  addvalue := ReadWizReg(_SHAR3)
    4:  addvalue := ReadWizReg(_SHAR4)
    5:  addvalue := ReadWizReg(_SHAR5)

  return addvalue

PUB SetSubnetMask(_maskbyte0, _maskbyte1, _maskbyte2, _maskbyte3)
''Writes the specified subnet mask to the WIZnet

  SubnetMask[0] := _maskbyte0
  SubnetMask[1] := _maskbyte1
  SubnetMask[2] := _maskbyte2
  SubnetMask[3] := _maskbyte3

  'Write data to WIZnet
  WriteWizReg(_SUBR0,SubnetMask[0])
  WriteWizReg(_SUBR1,SubnetMask[1])
  WriteWizReg(_SUBR2,SubnetMask[2])
  WriteWizReg(_SUBR3,SubnetMask[3])

PUB ReadSubnetMask(_maskbyte) : maskvalue
''Reads the specified subnet mask byte out of WIZnet

  'Read data from WIZnet
  case _maskbyte
    0:  maskvalue := ReadWizReg(_SUBR0)
    1:  maskvalue := ReadWizReg(_SUBR1)
    2:  maskvalue := ReadWizReg(_SUBR2)
    3:  maskvalue := ReadWizReg(_SUBR3)

  return maskvalue
                                     
PUB SetIPAddress(_addbyte0, _addbyte1, _addbyte2, _addbyte3)
''Writes the specified IP address to the WIZnet

  IPAddress[0] := _addbyte0
  IPAddress[1] := _addbyte1
  IPAddress[2] := _addbyte2
  IPAddress[3] := _addbyte3

  'Write data to WIZnet
  WriteWizReg(_SIPR0,IPAddress[0])
  WriteWizReg(_SIPR1,IPAddress[1])
  WriteWizReg(_SIPR2,IPAddress[2])
  WriteWizReg(_SIPR3,IPAddress[3])

PUB ReadIPAddress(_addbyte) : addvalue
''Reads the specified IP address byte out of WIZnet

  'Read data from WIZnet
  case _addbyte
    0:  addvalue := ReadWizReg(_SIPR0)
    1:  addvalue := ReadWizReg(_SIPR1)
    2:  addvalue := ReadWizReg(_SIPR2)
    3:  addvalue := ReadWizReg(_SIPR3)

  return addvalue

PRI WriteWizReg(_register, _databyte) | txbyte0, txbyte1, txbyte2, txbyte3
'Routine to bit bang write the SPI to the WIZnet chip
'Data is latched into WIZnet on the rising edge of the clock
'Note that SPIN is slow and this should be coded in ASM for best performance

  'prepare 32 bits to be sent with OP code, register and data
  txrxlong := (_WRITEOP << 24) | (_register << 8) | (_databyte & $0000_00FF)

  'ensure clock is in correct state
  OUTA[SCLKpin] := _LOW
  'begin data transmission
  OUTA[SSpin] := _LOW
    '32 bits in a transmission
    repeat 32
      OUTA[MOSIpin] := (txrxlong <-= 1) & 1
      'toggle the clock pin high, WIZnet reads data
      OUTA[SCLKpin] := _HIGH
      'toggle the clock pin low
      OUTA[SCLKpin] := _LOW
  'end data transmission
  OUTA[SSpin] := _HIGH

PRI ReadWizReg(_register) : readvalue
'Routine to bit bang read the SPI from the WIZnet chip
'Data is output from the WIZnet on the falling edge of the clock
'Note that SPIN is slow and this should be coded in ASM for best performance

  'prepare 32 bits to be sent with OP code, register and no data
  'note that a four bytes are transmitted, but while being transmitted bytes are read too
  txrxlong := (_READOP << 24) | (_register << 8) & $FFFF_FF00

  'ensure clock is in correct state
  OUTA[SCLKpin] := _LOW
  'begin data transmission
  OUTA[SSpin] := _LOW
    '32 bits in a transmission
    repeat 32
      OUTA[MOSIpin] := (txrxlong <-= 1) & 1
      'toggle the clock pin high, WIZnet reads data
      OUTA[SCLKpin] := _HIGH
      'read in the data from WIZnet one bit at a time while shifting the value over
      readvalue := (readvalue << 1) | ina[MISOpin]
     'toggle the clock pin low
      OUTA[SCLKpin] := _LOW
  'end data transmission
  OUTA[SSpin] := _HIGH

  'mask so only lower byte shows
  readvalue &= $FF

  return readvalue

PRI PauseMSec(Duration)
''Pause execution in milliseconds.
''Duration = number of milliseconds to delay
  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)

DAT                             'Assembly code and initialized variables
''None at this time

