''**************************************
''
''  WIZnet Test Driver Ver. 00.1
''
''  Timothy D. Swieter, E.I.
''  company name to go here
''
''  Updated: December 7, 2007
''
''Description:
''This program is used to test the driver for WIZnet W5100  
''
''reference:
''      W5100_Datasheet_v1_1_2.pdf (Datasheet for chip)
''      2711015Eady.pdf (Circuit Cellar Magazine Article)
''      http://bbs.circuitcellar.com/phpBB2/viewforum.php?f=30
''
''**************************************

CON
'***************************************
'  Hardware related settings
'***************************************
  _clkmode = xtal1 + pll16x                             'Use the PLL to multiple the external clock by 16
  _xinfreq = 5_000_000                                  'An external clock of 5MHz. is used (80MHz. operation)

'***************************************
'  System Definitions      
'***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3v DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0v DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode


  
  'Used for debug string construction, remove when debug remove
  CR            = 13                                    'Go to next line?                      
  FF            = 12                                    'Clear screen
  LF            = 10                                    'Go to next line
  
VAR               'Variables to be located here
'None at this time

OBJ               'Object declaration to be located here

  ETHERNET      : "TIM_WIZnet_Driver_Ver001.spin"
  Debug         : "PC_Debug.spin"                       'This object is used for debugging purposes only, remove before production

PUB MAIN | temp0, temp1, temp2, temp3, temp4, temp5, testlong, testbit


  ' Setup the Debug parameters so items can be viewed in Hyperterminal
  debug.start(460_800)                                  'start terminal
  PauseMSec(5000)                                       'Wait 5 second for me to switch applications to hyperterminal
  debug.str(string(FF, "Ethernet Tester", CR, LF, LF))  'print string

  'Setup the module for being used
  '                   ss, sclk, mosi, miso
  ETHERNET.InitWIZnet(0, 1, 2, 3)
  debug.str(string("Init", CR, LF))                     'print string
  PauseMSec(200)
  
  'software reset of the module
  ETHERNET.ResetWIZnet
  debug.str(string("Reset", CR, LF))                    'print string
  PauseMSec(1000)

  'configure the addressess
  ETHERNET.SetGatewayAddress(192,168,0,1)
  ETHERNET.SetMACAddress($C0,$A8,$00,$01,$00,$00)
  ETHERNET.SetSubnetMask(255,255,255,0)
  ETHERNET.SetIPAddress(192,168,0,32)
  debug.str(string("Address Set", CR, LF))              'print string

  ETHERNET.PingBlockDisable

  PauseMSec(5000)                                       'Wait 5 second

  repeat
    temp0 := ETHERNET.ReadIPAddress(0)
    temp1 := ETHERNET.ReadIPAddress(1)
    temp2 := ETHERNET.ReadIPAddress(2)
    temp3 := ETHERNET.ReadIPAddress(3)
    debug.str(string("IP Address: ")) 'print string
    debug.dec(temp0)
    debug.str(string("."))
    debug.dec(temp1)
    debug.str(string("."))
    debug.dec(temp2)
    debug.str(string("."))
    debug.dec(temp3)
    debug.str(string(CR, LF))
    PauseMSec(250)

    temp0 := ETHERNET.ReadMACAddress(0)
    temp1 := ETHERNET.ReadMACAddress(1)
    temp2 := ETHERNET.ReadMACAddress(2)
    temp3 := ETHERNET.ReadMACAddress(3)
    temp4 := ETHERNET.ReadMACAddress(4)
    temp5 := ETHERNET.ReadMACAddress(5)

    debug.str(string("MAC Address: ")) 'print string
    debug.hex(temp0,2)
    debug.str(string("."))
    debug.hex(temp1,2)
    debug.str(string("."))
    debug.hex(temp2,2)
    debug.str(string("."))
    debug.hex(temp3,2)
    debug.str(string("."))
    debug.hex(temp4,2)
    debug.str(string("."))
    debug.hex(temp5,2)
    debug.str(string(CR, LF))
    PauseMSec(250)

    temp0 := ETHERNET.ReadGatewayAddress(0)
    temp1 := ETHERNET.ReadGatewayAddress(1)
    temp2 := ETHERNET.ReadGatewayAddress(2)
    temp3 := ETHERNET.ReadGatewayAddress(3)
    
    debug.str(string("Gateway Address: ")) 'print string
    debug.dec(temp0)
    debug.str(string("."))
    debug.dec(temp1)
    debug.str(string("."))
    debug.dec(temp2)
    debug.str(string("."))
    debug.dec(temp3)
    debug.str(string(CR, LF))
    PauseMSec(250)

    temp0 := ETHERNET.ReadSubnetMask(0)
    temp1 := ETHERNET.ReadSubnetMask(1)
    temp2 := ETHERNET.ReadSubnetMask(2)
    temp3 := ETHERNET.ReadSubnetMask(3)
    
    debug.str(string("Subnet Mask: ")) 'print string
    debug.dec(temp0)
    debug.str(string("."))
    debug.dec(temp1)
    debug.str(string("."))
    debug.dec(temp2)
    debug.str(string("."))
    debug.dec(temp3)
    debug.str(string(CR, LF))
    PauseMSec(250)    

PRI PauseMSec(Duration)
''Pause execution in milliseconds.
''Duration = number of milliseconds to delay
  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)